/*
 * Copyright 2012 Smallcrew, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.smallcrew.rest.wadl;

import fr.smallcrew.rest.data.wadl.Resource;
import fr.smallcrew.rest.data.wadl.Resources;
import org.springframework.http.MediaType;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.mvc.condition.RequestMethodsRequestCondition;
import org.springframework.web.servlet.mvc.method.RequestMappingInfo;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

import javax.servlet.http.HttpServletRequest;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import static com.google.common.base.Throwables.propagate;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Maps.newHashMap;

public class ResourcesBuilder {
  public static final Comparator<Resource> RESOURCE_COMPARATOR = new Comparator<Resource>() {
    @Override
    public int compare(Resource r1, Resource r2) {
      return r1.getPath().compareTo(r2.getPath());
    }
  };

  private final RequestMappingHandlerMapping handlerMapping;
  private final HttpServletRequest request;
  private final Collection<MediaType> mediaTypes;

  private ResourcesBuilder(HttpServletRequest request, RequestMappingHandlerMapping handlerMapping,
                           Collection<MediaType> mediaTypes) {
    this.handlerMapping = handlerMapping;
    this.request = request;
    this.mediaTypes = mediaTypes;
  }

  public static ResourcesBuilder from(HttpServletRequest request, RequestMappingHandlerMapping handlerMapping,
                                      Collection<MediaType> mediaTypes) {
    return new ResourcesBuilder(request, handlerMapping, mediaTypes);
  }

  public Resources build() {
    Map<RequestMappingInfo, HandlerMethod> handlers = handlerMapping.getHandlerMethods();
    Map<String, Resource> resourceMap = newHashMap();
    for (RequestMappingInfo mapping : handlers.keySet()) {
      for (String uri : mapping.getPatternsCondition().getPatterns()) {
        if (!resourceMap.containsKey(uri)) {
          resourceMap.put(uri, newResource(uri));
        }
        java.lang.reflect.Method method = handlers.get(mapping).getMethod();
        RequestMethodsRequestCondition methodsCondition = mapping.getMethodsCondition();
        resourceMap.get(uri).getMethodOrResource().addAll(MethodBuilder.from(method, methodsCondition, mediaTypes).build());
      }
    }
    return newResources(newArrayList(resourceMap.values()));
  }

  private Resource newResource(String uri) {
    Resource resource = new Resource();
    resource.setPath(uri);
    return resource;
  }

  private Resources newResources(List<Resource> resourcesList) {
    Collections.sort(resourcesList, RESOURCE_COMPARATOR);
    Resources resources = new Resources();
    resources.setBase(newBase(request));
    for (Resource resource : resourcesList) {
      resources.getResource().add(resource);
    }
    return resources;
  }

  private String newBase(HttpServletRequest r) {
    try {
      return new URL(r.getScheme(), r.getServerName(), r.getServerPort(), r.getServletPath()).toString();
    } catch (Exception e) {
      throw propagate(e);
    }
  }
}
